@@ -6,7 +6,7 @@ use warnings::illegalproto ();
 use Moo ();
 use Web::Dispatch::Wrapper ();
 
-our $VERSION = '0.011';
+our $VERSION = '0.020';
 
 sub import {
   my ($class, $app_package) = @_;
@@ -30,6 +30,8 @@ sub _export_into {
   $INC{"${name}.pm"} = 'Set by "use Web::Simple;" invocation';
 }
 
+1;
+
 =head1 NAME
 
 Web::Simple - A quick and easy way to build simple web applications
@@ -40,7 +42,7 @@ Web::Simple - A quick and easy way to build simple web applications
   #!/usr/bin/env perl
 
   package HelloWorld;
-  use Web::Simple
+  use Web::Simple;
 
   sub dispatch_request {
     sub (GET) {
@@ -322,30 +324,15 @@ also match more than one part:
 and so on. To match an arbitrary number of parts, use -
 
   sub (/page/**) {
+    my ($self, $match) = @_;
 
-This will result in an element per /-separated part so matched. Note that
-you can do
+This will result in a single element for the entire match. Note that you can do
 
   sub (/page/**/edit) {
 
 to match an arbitrary number of parts up to but not including some final
 part.
 
-Finally,
-
-  sub (/foo/...) {
-
-Will match /foo/ on the beginning of the path -and- strip it. This is designed
-to be used to construct nested dispatch structures, but can also prove useful
-for having e.g. an optional language specification at the start of a path.
-
-Note that the '...' is a "maybe something here, maybe not" so the above
-specification will match like this:
-
-  /foo         # no match
-  /foo/        # match and strip path to '/'
-  /foo/bar/baz # match and strip path to '/bar/baz'
-
 Note: Since Web::Simple handles a concept of file extensions, * and **
 matchers will not by default match things after a final dot, and this
 can be modified by using *.* and **.* in the final position, i.e.:
@@ -355,6 +342,99 @@ can be modified by using *.* and **.* in the final position, i.e.:
   /**          matches /one/two.three    and captures "one/two"
   /**.*        matches /one/two.three    and captures "one/two.three"
 
+Finally,
+
+  sub (/foo/...) {
+
+Will match C</foo/> on the beginning of the path -and- strip it. This is
+designed to be used to construct nested dispatch structures, but can also prove
+useful for having e.g. an optional language specification at the start of a
+path.
+
+Note that the '...' is a "maybe something here, maybe not" so the above
+specification will match like this:
+
+  /foo         # no match
+  /foo/        # match and strip path to '/'
+  /foo/bar/baz # match and strip path to '/bar/baz'
+
+Almost the same,
+
+  sub (/foo...) {
+
+Will match on C</foo/bar/baz>, but also include C</foo>.  Otherwise it
+operates the same way as C</foo/...>.
+
+  /foo         # match and strip path to ''
+  /foo/        # match and strip path to '/'
+  /foo/bar/baz # match and strip path to '/bar/baz'
+
+Please note the difference between C<sub(/foo/...)> and C<sub(/foo...)>.  In
+the first case, this is expecting to find something after C</foo> (and fails to
+match if nothing is found), while in the second case we can match both C</foo>
+and C</foo/more/to/come>.  The following are roughly the same:
+
+  sub (/foo)   { 'I match /foo' },
+  sub (/foo/...) {
+    sub (/bar) { 'I match /foo/bar' },
+    sub (/*)   { 'I match /foo/{id}' },
+  }
+
+Versus
+
+  sub (/foo...) {
+    sub (~)    { 'I match /foo' },
+    sub (/bar) { 'I match /foo/bar' },
+    sub (/*)   { 'I match /foo/{id}' },
+  }
+
+You may prefer the latter example should you wish to take advantage of
+subdispatchers to scope common activities.  For example:
+
+  sub (/user...) {
+    my $user_rs = $schema->resultset('User');
+    sub (~) { $user_rs },
+    sub (/*) { $user_rs->find($_[1]) },
+  }
+
+You should note the special case path match C<sub (~)> which is only meaningful
+when it is contained in this type of path match. It matches to an empty path.
+
+=head4 Naming your patch matches
+
+Any */**/*.*/**.* match can be followed with :name to make it into a named
+match, so:
+
+  sub (/*:one/*:two/*:three/*:four) {
+    "I match /1/2/3/4 capturing { one => 1, two =>  2, three => 3, four => 4 }"
+  }
+  
+  sub (/**.*:allofit) {
+    "I match anything capturing { allofit => \$whole_path }"
+  }
+
+In the specific case of a simple single-* match, the * may be omitted, to
+allow you to write:
+
+  sub (/:one/:two/:three/:four) {
+    "I match /1/2/3/4 capturing { one => 1, two =>  2, three => 3, four => 4 }"
+  }
+
+=head4 C</foo> and C</foo/> are different specs
+
+As you may have noticed with the difference between C<sub(/foo/...)> and
+C<sub(/foo...)>, trailing slashes in path specs are significant. This is
+intentional and necessary to retain the ability to use relative links on
+websites. Let's demonstrate on this link:
+
+  <a href="bar">bar</a>
+
+If the user loads the url C</foo/> and clicks on this link, they will be
+sent to C</foo/bar>. However when they are on the url C</foo> and click this
+link, then they will be sent to C</bar>.
+
+This makes it necessary to be explicit about the trailing slash.
+
 =head3 Extension matches
 
   sub (.html) {
@@ -381,8 +461,7 @@ Query and body parameters can be match via
 
 The body spec will match if the request content is either
 application/x-www-form-urlencoded or multipart/form-data - the latter
-of which is required for uploads, which are now handled experimentally
-- see below.
+of which is required for uploads - see below.
 
 The param spec is elements of one of the following forms -
 
@@ -445,11 +524,7 @@ hashref style, the arrayref and single parameters will appear in C<@_> in the
 order you defined them in the protoype, but all hashrefs will merge into a
 single C<$params>, as in the example above.
 
-=head3 Upload matches (EXPERIMENTAL)
-
-Note: This feature is experimental. This means that it may not remain
-100% in its current form. If we change it, notes on updating your code
-will be added to the L</CHANGES BETWEEN RELEASES> section below.
+=head3 Upload matches
 
   sub (*foo=) { # param specifier can be anything valid for query or body
 
@@ -531,6 +606,45 @@ from subroutine prototypes, so this is equivalent to
 
   sub (GET+/user/*) {
 
+=head3 Accessing parameters via C<%_>
+
+If your dispatch specification causes your dispatch subroutine to receive
+a hash reference as its first argument, the contained named parameters
+will be accessible via C<%_>.
+
+This can be used to access your path matches, if they're named:
+
+  sub (GET + /foo/:path_part) {
+    [ 200,
+      ['Content-type' => 'text/plain'],
+      ["We are in $_{path_part}"],
+    ];
+  }
+
+Or, if your first argument would be a hash reference containing named
+query parameters:
+
+  sub (GET + /foo + ?:some_param=) {
+    [ 200,
+      ['Content-type' => 'text/plain'],
+      ["We received $_{some_param} as parameter"],
+    ];
+  }
+
+Of course this also works when all you are doing is slurping the whole set
+of parameters by their name:
+
+  sub (GET + /foo + ?*) {
+    [ 200,
+      ['Content-type' => 'text/plain'],
+      [exists($_{foo}) ? "Received a foo: $_{foo}" : "No foo!"],
+    ],
+  }
+
+Note that only the first hash reference will be avaialble via C<%_>. If
+you receive additional hash references, you will need to access them as
+usual.
+
 =head3 Accessing the PSGI env hash
 
 In some cases you may wish to get the raw PSGI env hash - to do this,
@@ -706,7 +820,7 @@ John Napiorkowski (jnap) <jjn1056@yahoo.com>
 
 Josh McMichael <jmcmicha@linus222.gsc.wustl.edu>
 
-Justin Hunter <justin.d.hunter@gmail.com>
+Justin Hunter (arcanez) <justin.d.hunter@gmail.com>
 
 Kjetil Kjernsmo <kjetil@kjernsmo.net>
 
@@ -718,9 +832,13 @@ nperez <nperez@cpan.org>
 
 Robin Edwards <robin.ge@gmail.com>
 
+Andrew Rodland (hobbs) <andrew@cleverdomain.org>
+
+Robert Sedlacek (phaylon) <r.sedlacek@shadowcat.co.uk>
+
 =head1 COPYRIGHT
 
-Copyright (c) 2010 the Web::Simple L</AUTHOR> and L</CONTRIBUTORS>
+Copyright (c) 2011 the Web::Simple L</AUTHOR> and L</CONTRIBUTORS>
 as listed above.
 
 =head1 LICENSE
@@ -729,5 +847,3 @@ This library is free software and may be distributed under the same terms
 as perl itself.
 
 =cut
-
-1;
